@@ -572,7 +572,7 @@ void Prot_handle_token( channel fd, int dummy, void *dummy_p )
 
         /* Deal with wrapping seq values (2^32) by triggering a membership by dropping the token */
 
-        if ( Memb_state() != EVS && Token->seq > MAX_WRAP_SEQUENCE_VALUE )
+        if ( Memb_state() != EVS && Token->seq > MAX_WRAP_SEQUENCE_VALUE )  /* NOTE: Token->seq is meaningless in EVS */
         {
                 Alarmp( SPLOG_WARNING, PROTOCOL, "Prot_handle_token: seq (%d) rollover; swallowing token to trigger membership\n", Token->seq );
                 goto END;
@@ -639,10 +639,7 @@ void Prot_handle_token( channel fd, int dummy, void *dummy_p )
                 }
         }
 
-        if ( Highest_seq < Token->seq ) 
-        {
-                Highest_seq = Token->seq;
-        }
+        if ( Memb_state() != EVS && Highest_seq < Token->seq ) Highest_seq = Token->seq;  /* NOTE: Token->seq is meaningless in EVS */
                 
         /* I don't want to process my next token until I have processed
          *  all bcast packets sent in the previous round, so I should give 
@@ -711,7 +708,7 @@ void Prot_handle_token( channel fd, int dummy, void *dummy_p )
          * numbers between the sequence on the last token this process sent
          * and the sequence on the token it just received may still be in
          * transit or not yet sent, so they should not be requested at this point*/
-        if ( FC_accelerated_ring() && Memb_state() != EVS ){
+        if ( FC_accelerated_ring() && Memb_state() != EVS ){  /* NOTE: Last_token->seq is meaningless in EVS */
                 max_rtr_seq = Last_token->seq;
         }else{
                 max_rtr_seq = Highest_seq;
@@ -769,7 +766,7 @@ void Prot_handle_token( channel fd, int dummy, void *dummy_p )
         }
         
         Token->proc_id = My.id;
-        if ( Memb_state() != EVS ) Token->seq = Highest_seq;
+        if ( Memb_state() != EVS ) Token->seq = Highest_seq; /* NOTE: Token->seq is meaningless in EVS */
 
         if ( Conf_leader( Memb_active_ptr() ) == My.id )
         {
@@ -1516,7 +1513,7 @@ static  void    Deliver_reliable_packets( int32 start_seq, int num_packets )
 {
         int             pack_entry;
         int             end_seq;
-        int             i;
+        int32           i;
 
         if ( Memb_state() == EVS ) return;
         if ( Prot_delivery_threshold > RELIABLE_TYPE ) return;
@@ -1550,7 +1547,7 @@ static  void    Deliver_agreed_packets()
         /* deliver all non-safe packets that are ordered and not delivered */
 
         int             pack_entry;
-        int             i;
+        int32           i;
 
         if ( My_aru <= Last_delivered ) return;
         if ( Memb_state() == EVS ) return;
@@ -1590,7 +1587,7 @@ void    Discard_packets()
         packet_body     *body_ptr;
         up_queue        *up_ptr;
         int             proc_index;
-        int             i;
+        int32           i;
 
         if ( Aru <= Last_discarded ) return;
         if ( Memb_state() == EVS )
